function expe_real_vincent
%EXPE_REAL_VINCENT Experiments on real data (from Vincent GB, with
%auto-occlusions)
%
% (c)2010, Florent Brunet

% RIS is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% RIS is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
% or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
% for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program; if not, write to the Free Software Foundation, Inc.,
% 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

close all;

%% Load the data
[corresp_2d_tpl corresp_2d_img tpl img cam_mat] = load_data(150);

a = [];
b = [];
for i = 1:size(corresp_2d_tpl,2)
    if ~((corresp_2d_img(1,i) >= 353) && (corresp_2d_img(1,i) <= 362) && (corresp_2d_img(2,i) >= 49) && (corresp_2d_img(1,i) <= 426))
        a = [a corresp_2d_tpl(:,i)];
        b = [b corresp_2d_img(:,i)];
    end
end
corresp_2d_tpl = a;
corresp_2d_img = b;

% figure(1);
% imshow(tpl);
% hold on;
% plot(corresp_2d_tpl(1,:), corresp_2d_tpl(2,:), '.');
% 
% figure(2);
% imshow(img);
% hold on;
% plot(corresp_2d_img(1,:), corresp_2d_img(2,:), '.');

% Display the template
figure(1);
imshow(tpl);
hold on;
plot(corresp_2d_tpl(1,:), corresp_2d_tpl(2,:), '.', 'Markersize', 28, 'MarkerFaceColor', [0 0 0.5], 'MarkerEdgeColor', [0 0 0.33]);
plot(corresp_2d_tpl(1,:), corresp_2d_tpl(2,:), 'b.', 'Markersize', 22);


figure(2);
imshow(img);
hold on;
plot(corresp_2d_img(1,:), corresp_2d_img(2,:), '.', 'Markersize', 28, 'MarkerFaceColor', [0 0 0.5], 'MarkerEdgeColor', [0 0 0.33]);
plot(corresp_2d_img(1,:), corresp_2d_img(2,:), 'b.', 'Markersize', 22);
set(gca, 'xlim', [90 580], 'ylim', [35 465]);




%% (P1) Perriollat

% compute lengts on the template and apply the corrective term to deal with
% real distances
corTerm = 1;

[depth,linkTo,bounds3D,linkAngle] = ...
    ris_perriollat_bounding( ...
        corresp_2d_tpl, ...
        corresp_2d_img, ...
        cam_mat, ...
        corTerm);
    
% Display the upper bounds
% figure(3);
% plot3(bounds3D(1,:), bounds3D(2,:), bounds3D(3,:), '.');
% daspect([1 1 1]);
figure(3);
%plot3(pts_3d_r1(1,:), pts_3d_r1(2,:), pts_3d_r1(3,:), '.');
plot3(bounds3D(1,:), bounds3D(2,:), bounds3D(3,:), '.', 'Markersize', 22, 'MarkerFaceColor', [0.5 0 0], 'MarkerEdgeColor', [0.55 0 0]);
hold on;
plot3(bounds3D(1,:), bounds3D(2,:), bounds3D(3,:), 'r.', 'Markersize', 10);
daspect([1 1 1]);
view(207,12);
axis off

% Reproject the points
tmp = cam_mat * [bounds3D ; ones(1, size(bounds3D,2))];
tmp = tmp(1:2,:) ./ repmat(tmp(3,:),2,1);


figure(30);
imshow(img);
hold on;
plot(tmp(1,:), tmp(2,:), '.', 'Markersize', 28, 'MarkerFaceColor', [0.5 0 0], 'MarkerEdgeColor', [0.33 0 0]);
plot(tmp(1,:), tmp(2,:), 'r.', 'Markersize', 22);
set(gca, 'xlim', [90 580], 'ylim', [35 465]);


%% (R1) Richard's one

% pts_3d_r1 = ris_richard_errimg( ...
%     corresp_2d_tpl, corresp_2d_img, cam_mat, 0.4, 0.6);
% save('pts_3d_r1.mat', 'pts_3d_r1');
load('pts_3d_r1.mat');

figure(4);
%plot3(pts_3d_r1(1,:), pts_3d_r1(2,:), pts_3d_r1(3,:), '.');
plot3(pts_3d_r1(1,:), pts_3d_r1(2,:), pts_3d_r1(3,:), '.', 'Markersize', 22, 'MarkerFaceColor', [0.5 0 0], 'MarkerEdgeColor', [0.55 0 0]);
hold on;
plot3(pts_3d_r1(1,:), pts_3d_r1(2,:), pts_3d_r1(3,:), 'r.', 'Markersize', 10);
daspect([1 1 1]);
view(207,12);
axis off




cam = cam_mat;
%cam(:,4) = 0;
repro = cam * [pts_3d_r1 ; ones(1, size(pts_3d_r1,2))];
repro_r1 = repro(1:2,:) ./ repmat(repro(3,:), 2, 1);
%figure(2);
%plot(repro_r1(1,:), repro_r1(2,:), 'g*');



tmp = cam * [pts_3d_r1 ; ones(1,size(pts_3d_r1,2))];
tmp = tmp(1:2,:) ./ repmat(tmp(3,:),2,1);

figure(40);
imshow(img);
hold on;
plot(tmp(1,:), tmp(2,:), '.', 'Markersize', 28, 'MarkerFaceColor', [0.5 0 0], 'MarkerEdgeColor', [0.33 0 0]);
plot(tmp(1,:), tmp(2,:), 'r.', 'Markersize', 22);
set(gca, 'xlim', [90 580], 'ylim', [35 465]);

return

%% (B1) B-spline fitted to R1
EPS = 1.0e-3;
n_ctrlpts = 16;
umin = min(corresp_2d_tpl(1,:)) - EPS;
umax = max(corresp_2d_tpl(1,:)) + EPS;
nptsu = n_ctrlpts;
vmin = min(corresp_2d_tpl(2,:)) - EPS;
vmax = max(corresp_2d_tpl(2,:)) + EPS;
nptsv = n_ctrlpts;
bbs = bbs_create(umin, umax, nptsu, vmin, vmax, nptsv, 3);

colmat = bbs_coloc(bbs, corresp_2d_tpl(1,:), corresp_2d_tpl(2,:));
lbds = 0.1*ones(nptsv-3, nptsu-3);
bendmat = bbs_bending(bbs, lbds);

ctrlpts = (colmat'*colmat + bendmat) \ (colmat' * pts_3d_r1');
ctrlpts = ctrlpts';

% Display the initial B-spline
figure(5);
dnu = 7*2+1;
dnv = 9*2+1;
[du,dv] = meshgrid(linspace(umin,umax,dnu), linspace(vmin,vmax,dnv));
pts_3d_b1 = bbs_eval(bbs, ctrlpts, du, dv);
mesh(reshape(pts_3d_b1(1,:), dnv, dnu), ...
     reshape(pts_3d_b1(2,:), dnv, dnu), ...
     reshape(pts_3d_b1(3,:), dnv, dnu), ...
     'facecolor', 'none', 'edgecolor', 'k', 'linewidth', 2);
daspect([1 1 1]);
axis off;
view(207,12);


% Reprojects the mesh in the image
figure(6);
imshow(img);
hold on;
tmp = cam * [pts_3d_b1 ; ones(1,dnu*dnv)];
tmp = tmp(1:2,:) ./ repmat(tmp(3,:),2,1);
mesh(reshape(tmp(1,:), dnv, dnu), ...
     reshape(tmp(2,:), dnv, dnu), ...
     0.1*ones(dnv, dnu), 'facecolor', 'none', 'edgecolor', 'k', 'linewidth', 3);
mesh(reshape(tmp(1,:), dnv, dnu), ...
     reshape(tmp(2,:), dnv, dnu), ...
     0.1*ones(dnv, dnu), 'facecolor', 'none', 'edgecolor', [1 1 1], 'linewidth', 2);
set(gca, 'xlim', [90 580], 'ylim', [35 465]);

%plot3(repro_r1(1,:), repro_r1(2,:), ones(size(repro_r1(1,:))), '.', 'Markersize', 28, 'MarkerFaceColor', [0 0 0.5], 'MarkerEdgeColor', [0 0 0.33]);
%plot3(repro_r1(1,:), repro_r1(2,:), ones(size(repro_r1(1,:))), 'b.', 'Markersize', 22);




%% (B2) Refined B-spline

% Initial parameters
pts_3d_b1 = bbs_eval(bbs, ctrlpts, corresp_2d_tpl(1,:), corresp_2d_tpl(2,:));
param_init = [vect(sqrt(sum(pts_3d_b1.^2))) ; vect(ctrlpts')];

% Optimization
% ctrlpts_b2 = ris_jacob_inext_depth_jacob( ...
%     corresp_2d_tpl, corresp_2d_img, cam, bbs, param_init, 37, 10, 10);
% save('ctrlpts_b2.mat', 'ctrlpts_b2');
load('ctrlpts_b2.mat');

% Display the refined B-spline
figure(7);
[du,dv] = meshgrid(linspace(umin,umax,dnu), linspace(vmin,vmax,dnv));
pts_3d_b2 = bbs_eval(bbs, ctrlpts_b2, du, dv);
mesh(reshape(pts_3d_b2(1,:), dnv, dnu), ...
     reshape(pts_3d_b2(2,:), dnv, dnu), ...
     reshape(pts_3d_b2(3,:), dnv, dnu), ...
     'facecolor', 'none', 'edgecolor', 'k', 'linewidth', 2);
daspect([1 1 1]);
axis off;
view(207,12);
 
% Reprojects the mesh in the image
figure(8);
imshow(img);
hold on;
tmp = cam * [pts_3d_b2 ; ones(1,dnu*dnv)];
tmp = tmp(1:2,:) ./ repmat(tmp(3,:),2,1);
mesh(reshape(tmp(1,:), dnv, dnu), ...
     reshape(tmp(2,:), dnv, dnu), ...
     0.1*ones(dnv, dnu), 'facecolor', 'none', 'edgecolor', 'k', 'linewidth', 3);
mesh(reshape(tmp(1,:), dnv, dnu), ...
     reshape(tmp(2,:), dnv, dnu), ...
     0.1*ones(dnv, dnu), 'facecolor', 'none', 'edgecolor', [1 1 1], 'linewidth', 2);
 
set(gca, 'xlim', [90 580], 'ylim', [35 465]);








%% Salzmann's method

% Initialization for the Salzmann's method
n_corresp = size(corresp_2d_tpl,2);
npts_mesh = 10;
[tri tri_x tri_y tri_z] = ris_create_tri_mesh(umin, umax, npts_mesh, vmin, vmax, npts_mesh);

% Convert the template points into barycentric coordinates
[b1 b2 b3 ind_tri] = ris_cart_to_bary( ...
    tri, tri_x, tri_y, tri_z, ... 
    corresp_2d_tpl(1,:), corresp_2d_tpl(2,:), ones(size(corresp_2d_tpl(2,:))));

% Build the inextensibility pairs
thres_dist = 1.25 * sqrt(2*(max(umax-umin, vmax-vmin) / (npts_mesh - 1))^2);
[inext_pairs dist_pairs] = ris_make_inext_pairs(tri_x, tri_y, thres_dist);

% Reconstruction
% (x,y,z) are the Cartesian coordinates of the mesh _vertices_
[~, x, y, z] = ris_salzmann_err( ...
                        tri, npts_mesh^2, ...
                        ind_tri, b1, b2, b3, ...
                        corresp_2d_img, ...
                        cam_mat, ...
                        inext_pairs, dist_pairs, ...
                        ones(1,n_corresp), 5);

% Display the Salzmann's mesh
figure(10);
trimesh(tri, x, y, z, 'facecolor', 'none', 'edgecolor', 'k', 'linewidth', 2);
daspect([1 1 1]);
axis off;
view(207,12);
daspect([1 1 1]);

% Reprojects the mesh in the image
figure(11);
[xx yy zz] = bary_to_cart(tri, x, y, z, ind_tri, b1, b2, b3);
pts_3d_s = [x ; y ; z];
imshow(img);
hold on;
tmp = cam * [pts_3d_s ; ones(1,size(pts_3d_s,2))];
tmp = tmp(1:2,:) ./ repmat(tmp(3,:),2,1);
trimesh(tri,tmp(1,:), ...
     tmp(2,:), ...
     0.1*ones(size(tmp(1,:))), 'facecolor', 'none', 'edgecolor', 'k', 'linewidth', 3);
trimesh(tri,tmp(1,:), ...
     tmp(2,:), ...
     0.1*ones(size(tmp(1,:))), 'facecolor', 'none', 'edgecolor', [1 1 1], 'linewidth', 2);
 
set(gca, 'xlim', [90 580], 'ylim', [35 465]);
    
    
function [corresp_2d_tpl corresp_2d_img tpl img cam_mat] = load_data(i_img)

load('data_vincent/mesh.mat');

corresp = reshape(a(i_img,:), 15, 18, 2);
corresp_2d_img = [vect(corresp(:,:,1))' ; vect(corresp(:,:,2))'];
path_img = sprintf('data_vincent/video/%08d.png', i_img);
img = imread(path_img);


load('data_vincent/paperSelfOcclusionsData/templatePSO.mat');

tpl = imTemplate;

stepMesh=20;
sizeImgTexture = size(imTemplate);
xMesh=fix(linspace(1,sizeImgTexture(2),sizeImgTexture(2)/stepMesh));
yMesh=fix(linspace(1,sizeImgTexture(1),sizeImgTexture(1)/stepMesh));
[XMesh,YMesh]=meshgrid(xMesh,yMesh);
corresp_2d_tpl = [XMesh(:),YMesh(:)]';

load('data_vincent/paperSelfOcclusionsData/camera.mat');
cam_mat = camera;


function [x y z] = bary_to_cart(tri, tx, ty, tz, ind_tri, b1, b2, b3)
% Evaluate the 3D position of the data points
for i = 1:numel(b1)
    i1 = tri(ind_tri(i), 1);
    i2 = tri(ind_tri(i), 2);
    i3 = tri(ind_tri(i), 3);
    x(i) = b1(i) * tx(i1) + b2(i) * tx(i2) + b3(i) * tx(i3); %#ok<AGROW>
    y(i) = b1(i) * ty(i1) + b2(i) * ty(i2) + b3(i) * ty(i3); %#ok<AGROW>
    z(i) = b1(i) * tz(i1) + b2(i) * tz(i2) + b3(i) * tz(i3); %#ok<AGROW>
end