function [inext_pairs dist_pairs] = ris_make_inext_pairs(x, y, thres)
%RIS_MAKE_INEXT_PAIRS Make pairs of points for inextensibility constraints.
%
% SYNTAX
%  [inext_pairs dist_pairs] = ris_make_inext_pairs(x, y, thres)
%
% INPUT ARGUMENTS
%  - x, y [matrices]: coordinates of the template points
%  - thres [scalar]: threshold distance above which pairs of point are not
%     built.
%
% OUTPUT ARGUMENTS
%  - inext_pairs [2 x n_inext matrix]: indices (pointing into x and y) of
%     the pair of points. An actual pair is made of these two points:
%     * (x(inext_pairs(1,i)), y(inext_pairs(1,i)))
%     * (x(inext_pairs(2,i)), y(inext_pairs(2,i)))
%  - dist_pairs [1 x inext matrix]: distance between the two points of the
%     i-th pair.
%
% (c)2010, Florent Brunet

% RIS is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% RIS is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
% or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
% for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program; if not, write to the Free Software Foundation, Inc.,
% 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

npts = numel(x);


vec_dist = pdist([x(:) y(:)]);
[ind_j ind_i] = meshgrid(1:npts);
ind_i = squareform(tril(ind_i, -1));
ind_j = squareform(tril(ind_j, -1));
ind = find(vec_dist <= thres);
inext_pairs = [ind_i(ind) ; ind_j(ind)];

if nargout >= 2
    dist_pairs = vec_dist(ind);
end
