function [At, bt, ct, K] = ris_readable_to_sedumi(f, A, c, b, d)
%RIS_READABLE_TO_SEDUMI Transforms a set of understandable constraints to
%parameters for SeDuMi.
%
% SYNTAX
%  [At, bt, ct, K] = ris_readable_to_sedumi(b, Ai, ci, bi, di)
%
% DESCRIPTION
%  Creates the input parameters (At, bt, ct, K) for the function "sedumi"
%  that solves this problem:
%   minimize f' * x
%   s.t.     || A{i}*x + c{i} || <= b{i}' * x + d{i},  i=1,...,q
%
%  Mainly based on the document "Wu-Sheng Lu. Use SeDuMi to Solve LP, SDP
%  and SOCP Problems: Remarks and Example. 2007", particularly the section
%  "II. SOCP Problems".
%
% INPUT ARGUMENTS
%
% OUTPUT ARGUMENTS
%
% (c)2010, Florent Brunet

% RIS is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% RIS is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
% or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
% for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program; if not, write to the Free Software Foundation, Inc.,
% 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

% Number of cone constraints
q = numel(A);

bt = -f;

At = [];
ct = [];
K.q = [];
for i = 1:q
    At = [At -b{i} -A{i}']; %#ok<AGROW>
    ct = [ct ; d{i} ; c{i}]; %#ok<AGROW>
    K.q = [K.q 1+size(A{i},1)];
end
