function x = ris_test_gauss_seidel(A, x, b)
%RIS_TEST_GAUSS_SEIDEL Test of the Gauss Seidel algorithm.
%
% Syntax
%  x = ris_test_gauss_seidel(A, x, b)
%
% Description
%  Solves the linear system Ax=b.
%
% Input arguments
%
%  - A [n x n matrix]
%
%  - x [n x 1 vector]: initial solution.
%
%  - b [n x 1 vector]: right hand side of the linear system.
%
% Output arguments
%
%  - x [n x 1 vector]: solution of the linear system  Ax=b.
%
% (c)2010, Florent Brunet

% RIS is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% RIS is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
% or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
% for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program; if not, write to the Free Software Foundation, Inc.,
% 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

% Size of the system
n = size(A,1);

% Paramaters for the stopping criterion
EPS_NORM = 1.0e-6;
MAX_ITER = 1000;

previous_x = x;
n_iter = 1;

% Gauss-Seidel iterations
while true
    for i = 1:n
        a1 = A(i,1:i-1);
        a2 = A(i,i+1:end);
        x1 = x(1:i-1);
        x2 = x(i+1:end);
        x(i) = (b(i) - a1*x1 - a2*x2) / A(i,i);
    end
    
    norm_x = norm(previous_x-x);
    if (norm_x <= EPS_NORM) || (n_iter >= MAX_ITER)
        return
    end
    
    previous_x = x;
    n_iter = n_iter + 1;
    fprintf('(%d)   ||x_{i-1}-x_i|| = %f\n', n_iter, norm_x);
end
