function test_generate_inext
%TEST_GENERATE_INEXT Test that generates random inextensible meshes.

% RIS is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% RIS is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
% or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
% for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program; if not, write to the Free Software Foundation, Inc.,
% 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

close all;

nptsx = 10;
xmin = 1;
xmax = nptsx;

nptsy = 6;
ymin = 1;
ymax = nptsy;

[tri tri_x tri_y tri_z] = ris_create_tri_mesh(xmin, xmax, nptsx, ymin, ymax, nptsy);

trimesh(tri, tri_x, tri_y, tri_z, 'facecolor', 'none', 'edgecolor', 'b');
daspect([1 1 1]);
view(0, 90);
grid off;

[pairs distp] = ris_make_inext_pairs(tri_x, tri_y, 2.1);

figure(2);
for i = 1:size(pairs,2)
    plot([tri_x(pairs(1,i)) tri_x(pairs(2,i))], [tri_y(pairs(1,i)) tri_y(pairs(2,i))]);
    hold on;
end
daspect([1 1 1]);

%rand('twister', 2);

%tri_x = tri_x + rand(size(tri_x));
%tri_y = tri_y + rand(size(tri_y));
%tri_z = tri_z + rand(size(tri_z));
perm = randperm(size(tri_z));
i_p1 = perm(1);
tri_z(i_p1) = 1+4*rand(1)-2;
i_p2 = perm(2);
tri_z(i_p2) = 1+4*rand(1)-2;
%tri_z(30) = 10;

figure(3);
trimesh(tri, tri_x, tri_y, tri_z, 'facecolor', 'none', 'edgecolor', 'b');

n_iter = 1;
old_tri_x = tri_x;
old_tri_y = tri_y;
old_tri_z = tri_z;
while true
    for i = 1:size(pairs,2)
        i1 = pairs(1,i);
        i2 = pairs(2,i);
        
        if (i2 == i_p1) || (i2 == i_p2)
            break
        end
        
        pt1 = [tri_x(i1) tri_y(i1) tri_z(i1)]';
        pt2 = [tri_x(i2) tri_y(i2) tri_z(i2)]';
        
        u = pt2 - pt1;
        cur_dist = norm(u);
        u = u / cur_dist;
        
        tmp = pt1 + distp(i)*u;
        tri_x(i2) = tmp(1);
        tri_y(i2) = tmp(2);
        
        
        
        tri_z(i2) = tmp(3);
    end
    
    norm_step = norm([old_tri_x old_tri_y old_tri_z] - [tri_x tri_y tri_z]);
    
    fprintf('[%d] norm_step=%f\n', n_iter, norm_step);
    
    if norm_step < 1.0e-4
        break
    end
    n_iter = n_iter + 1;
    old_tri_x = tri_x;
    old_tri_y = tri_y;
    old_tri_z = tri_z;
end

figure(4);
trimesh(tri, tri_x, tri_y, tri_z, 'facecolor', 'none', 'edgecolor', 'b');
daspect([1 1 1]);
    
actual_dist = zeros(1, size(pairs,2));
for i = 1:size(pairs,2)
    i1 = pairs(1,i);
    i2 = pairs(2,i);

    pt1 = [tri_x(i1) tri_y(i1) tri_z(i1)]';
    pt2 = [tri_x(i2) tri_y(i2) tri_z(i2)]';

    actual_dist(i) = norm(pt2 - pt1);
end

mean(abs(distp-actual_dist))
std(abs(distp-actual_dist))
max(abs(distp-actual_dist))
min(abs(distp-actual_dist))