function test_ris_jacob_inext
%TEST_RIS_JACOB_INEXT Test of the "ris_jacob_inext" function (inextensible
%surface reconstruction using the "jacobian" constraints).
%
% (c)2010, Florent Brunet

% RIS is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% RIS is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
% or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
% for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program; if not, write to the Free Software Foundation, Inc.,
% 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

close all;

rand('twister', 29);
%rand('twister', 39);
randn('state', 2);

% Generate a random piece of paper (without auto-occultations)
sz_grid = 100;
%[grid_2d_tpl grid_2d_img grid_3d cam_mat] = ris_rand_paper(sz_grid, true);
%save paper_jacob_inext.mat grid_2d_tpl grid_2d_img grid_3d cam_mat
load paper_jacob_inext.mat

% Extract some point corespondences
n_corresp = 150;
[corresp_2d_tpl corresp_2d_img corresp_3d] = ...
    ris_extract_corresp(n_corresp, grid_2d_tpl, grid_2d_img, grid_3d);

% Add noise
[corresp_2d_tpl corresp_2d_img] = ...
    ris_add_noise(corresp_2d_tpl, corresp_2d_img, 'gaussian', 1);

figure(1);
mesh(reshape(grid_3d(1,:), sz_grid, sz_grid), ...
     reshape(grid_3d(2,:), sz_grid, sz_grid), ...
     reshape(grid_3d(3,:), sz_grid, sz_grid), ...
     'facecolor', 'none', 'edgecolor', 0.5*ones(1,3));
daspect([1 1 1]);
hold on;
plot3(corresp_3d(1,:), corresp_3d(2,:), corresp_3d(3,:), 'r.');

figure(2);
plot(corresp_2d_tpl(1,:), corresp_2d_tpl(2,:), 'r.');
daspect([1 1 1]);

figure(3);
plot(corresp_2d_img(1,:), corresp_2d_img(2,:), 'r.');
daspect([1 1 1]);

%% Initial sparse solution
pts_3d = ris_richard_errimg(corresp_2d_tpl, corresp_2d_img, ...
                           cam_mat, 0.2, 0.8);
save init_jacob_inext.mat pts_3d
load init_jacob_inext.mat 

figure(1);
hold on;
plot3(pts_3d(1,:), pts_3d(2,:), pts_3d(3,:), 'bo');

%% Initial smooth solution
EPS = 1.0e-3;
umin = min(-100,min(corresp_2d_tpl(1,:))) - EPS;
umax = max(100,max(corresp_2d_tpl(1,:))) + EPS;
nptsu = 10;
vmin = min(-100,min(corresp_2d_tpl(2,:))) - EPS;
vmax = max(100,max(corresp_2d_tpl(2,:))) + EPS;
nptsv = 10;
bbs = bbs_create(umin, umax, nptsu, vmin, vmax, nptsv, 3);

colmat = bbs_coloc(bbs, corresp_2d_tpl(1,:), corresp_2d_tpl(2,:));
lbds = 0.1*ones(nptsv-3, nptsu-3);
bendmat = bbs_bending(bbs, lbds);

ctrlpts = (colmat'*colmat + bendmat) \ (colmat' * pts_3d');
ctrlpts = ctrlpts';

dn = 100;
[du dv] = meshgrid(linspace(umin,umax,dn), linspace(vmin,vmax,dn));
init_3d = bbs_eval(bbs, ctrlpts, du, dv);

figure(1);
hold on;
mesh(reshape(init_3d(1,:), dn, dn), ...
     reshape(init_3d(2,:), dn, dn), ...
     reshape(init_3d(3,:), dn, dn), ...
     'facecolor', 'none', 'edgecolor', 'r');
 
pause

%% Jacobian reconstruction
sz_inext = 20;
lbd_inext = 10;
ctrlpts_opt = ris_jacob_inext(corresp_2d_tpl, corresp_2d_img, ...
                              cam_mat, ...
                              bbs, ctrlpts(:), ...
                              sz_inext, lbd_inext);
                          
opt_3d = bbs_eval(bbs, ctrlpts_opt, du, dv);

figure(1);
hold on;
mesh(reshape(opt_3d(1,:), dn, dn), ...
     reshape(opt_3d(2,:), dn, dn), ...
     reshape(opt_3d(3,:), dn, dn), ...
     'facecolor', 'none', 'edgecolor', 'k');