function test_ris_jacob_inext_depth
%TEST_RIS_JACOB_INEXT_DEPTH Test of the "ris_jacob_inext_depth" function 
%(inextensible surface reconstruction using the "jacobian" constraints and
%a 3D error)
%
% (c)2010, Florent Brunet

% RIS is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% RIS is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
% or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
% for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program; if not, write to the Free Software Foundation, Inc.,
% 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

close all;

%{
rand('twister', 29);
%rand('twister', 39);
randn('state', 2);

% Generate a random piece of paper (without auto-occultations)
sz_grid = 100;
%[grid_2d_tpl grid_2d_img grid_3d cam_mat] = ris_rand_paper(sz_grid, true);
%save paper_jacob_inext.mat grid_2d_tpl grid_2d_img grid_3d cam_mat
load paper_jacob_inext.mat

% Extract some point corespondences
n_corresp = 150;
[corresp_2d_tpl corresp_2d_img corresp_3d] = ...
    ris_extract_corresp(n_corresp, grid_2d_tpl, grid_2d_img, grid_3d);

% Add noise
[corresp_2d_tpl corresp_2d_img] = ...
    ris_add_noise(corresp_2d_tpl, corresp_2d_img, 'gaussian', 1);

figure(1);
mesh(reshape(grid_3d(1,:), sz_grid, sz_grid), ...
     reshape(grid_3d(2,:), sz_grid, sz_grid), ...
     reshape(grid_3d(3,:), sz_grid, sz_grid), ...
     'facecolor', 'none', 'edgecolor', 0.5*ones(1,3));
daspect([1 1 1]);
hold on;
plot3(corresp_3d(1,:), corresp_3d(2,:), corresp_3d(3,:), 'r.');

figure(2);
plot(corresp_2d_tpl(1,:), corresp_2d_tpl(2,:), 'r.');
daspect([1 1 1]);

figure(3);
plot(corresp_2d_img(1,:), corresp_2d_img(2,:), 'r.');
daspect([1 1 1]);

%% Initial sparse solution
pts_3d = ris_richard_errimg(corresp_2d_tpl, corresp_2d_img, ...
                           cam_mat, 0.2, 0.8);
save init_jacob_inext.mat pts_3d
load init_jacob_inext.mat 

figure(1);
hold on;
plot3(pts_3d(1,:), pts_3d(2,:), pts_3d(3,:), 'bo');

% Depths of the initial 3D points
depths_init = sqrt(sum(pts_3d.^2));

%% Initial smooth solution
EPS = 1.0e-3;
umin = min(-100,min(corresp_2d_tpl(1,:))) - EPS;
umax = max(100,max(corresp_2d_tpl(1,:))) + EPS;
nptsu = 12;
vmin = min(-100,min(corresp_2d_tpl(2,:))) - EPS;
vmax = max(100,max(corresp_2d_tpl(2,:))) + EPS;
nptsv = 12;
bbs = bbs_create(umin, umax, nptsu, vmin, vmax, nptsv, 3);

colmat = bbs_coloc(bbs, corresp_2d_tpl(1,:), corresp_2d_tpl(2,:));
lbds = 0.1*ones(nptsv-3, nptsu-3);
bendmat = bbs_bending(bbs, lbds);

ctrlpts = (colmat'*colmat + bendmat) \ (colmat' * pts_3d');
ctrlpts = ctrlpts';

dn = 100;
[du dv] = meshgrid(linspace(umin,umax,dn), linspace(vmin,vmax,dn));
init_3d = bbs_eval(bbs, ctrlpts, du, dv);

figure(1);
hold on;
mesh(reshape(init_3d(1,:), dn, dn), ...
     reshape(init_3d(2,:), dn, dn), ...
     reshape(init_3d(3,:), dn, dn), ...
     'facecolor', 'none', 'edgecolor', 'r');
 

%% Jacobian reconstruction
sz_inext = 25;
lbd_inext = 10;
lbd_bend = 0.001;
tic
ctrlpts_opt = ris_jacob_inext_depth_jacob(corresp_2d_tpl, corresp_2d_img, ...
                                  cam_mat, ...
                                  bbs, [depths_init(:) ; vect(ctrlpts')], ...
                                  sz_inext, lbd_inext, lbd_bend);
toc
%save out.mat ctrlpts_opt
opt_3d = bbs_eval(bbs, ctrlpts_opt, du, dv);

figure(1);
hold on;
mesh(reshape(opt_3d(1,:), dn, dn), ...
     reshape(opt_3d(2,:), dn, dn), ...
     reshape(opt_3d(3,:), dn, dn), ...
     'facecolor', 'none', 'edgecolor', 'k');

save out.mat
%}

load out.mat


%{
%% Gaussian curvature
n_pts = 1000;
u = 200*rand(1,n_pts)-100;
v = 200*rand(1,n_pts)-100;

all_kappa_opt = gaussian_curvature(bbs, ctrlpts_opt, u, v);
all_kappa_init = gaussian_curvature(bbs, ctrlpts, u, v); %#ok<NODEF>

ampx = max(ctrlpts(1,:))-min(ctrlpts(1,:));
ampy = max(ctrlpts(2,:))-min(ctrlpts(2,:));
ampz = max(ctrlpts(3,:))-min(ctrlpts(3,:));
ctrlpts_rand = [ampx*rand(1,size(ctrlpts,2))+min(ctrlpts(1,:)) ; 
    ampy*rand(1,size(ctrlpts,2))+min(ctrlpts(2,:))
    ampz*rand(1,size(ctrlpts,2))+min(ctrlpts(3,:))];

all_kappa_rand = gaussian_curvature(bbs, ctrlpts_rand, u, v);

m_init = mean(abs(all_kappa_init)) %#ok<NOPRT>
m_opt = mean(abs(all_kappa_opt)) %#ok<NOPRT>
m_rand = mean(abs(all_kappa_rand)) %#ok<NOPRT>

m_init/m_opt %#ok<NOPRT>
m_rand/m_init %#ok<NOPRT>
m_rand/m_opt %#ok<NOPRT>
%}

%% Pseudo geodesics
n_pts = 100000;
u1 = 200*rand(1,n_pts)-100;
v1 = 200*rand(1,n_pts)-100;
u2 = 200*rand(1,n_pts)-100;
v2 = 200*rand(1,n_pts)-100;

% Check the geodesic approximation
%{
all_n = 2:200;

all_geod = [];
for n = all_n
    [eucl, geod] = geodesics(bbs, ctrlpts_opt, u1, v1, u2, v2, n);
    all_geod = [all_geod geod];
end

close all;
plot(all_n, all_geod);
hold on;
plot([all_n(1) all_n(end)], [eucl eucl], 'r');
ylim = get(gca, 'ylim');
ylim(1) = 0;
set(gca, 'ylim', ylim);
%}

[all_eucl, all_geod] = geodesics(bbs, ctrlpts, u1, v1, u2, v2, 200);

figure(1);
plot(all_eucl, all_geod, '.');
daspect([1 1 1]);
xlim = get(gca, 'xlim');
ylim = get(gca, 'ylim');
hold on;
plot(xlim, ylim, 'k--', 'linewidth', 2);
xlabel('Template distance (mm)');
ylabel('Surface geodesic length (mm)');

figure(2);
hist(all_eucl-all_geod, 100);
xlabel('(Euclidean distance [template]) - (geodesic length [3D surface])');

figure(3);
hist((all_eucl-all_geod)./all_eucl, 100);
xlabel('(Euclidean distance [template]) - (geodesic length [3D surface])');

function [all_eucl all_geod] = geodesics(bbs, ctrlpts, u1, v1, u2, v2, n)
n_pts = numel(u1);
all_eucl = sqrt((u1(:)-u2(:)).^2 + (v1(:)-v2(:)).^2)';
all_geod = zeros(1, n_pts);
for i = 1:n_pts
    all_geod(i) = one_geodesic(bbs, ctrlpts, u1(i), v1(i), u2(i), v2(i), n);
end

function geod = one_geodesic(bbs, ctrlpts, u1, v1, u2, v2, n)
all_u = linspace(u1, u2, n);
all_v = linspace(v1, v2, n);
val = bbs_eval(bbs, ctrlpts, all_u, all_v);
tmp = sqrt(sum(([val zeros(3,1)] - [zeros(3,1) val]).^2));
geod = sum(tmp(2:end-1));


function all_kappa = gaussian_curvature(bbs, ctrlpts, u, v)

n_pts = numel(u);

val_u = bbs_eval(bbs, ctrlpts, u, v, 1, 0);
val_v = bbs_eval(bbs, ctrlpts, u, v, 0, 1);
val_uu = bbs_eval(bbs, ctrlpts, u, v, 2, 0);
val_uv = bbs_eval(bbs, ctrlpts, u, v, 1, 1);
val_vv = bbs_eval(bbs, ctrlpts, u, v, 0, 2);

all_kappa = zeros(1,n_pts);
for i = 1:n_pts
    u = val_u(:,i);
    v = val_v(:,i);
    uu = val_uu(:,i);
    uv = val_uv(:,i);
    vv = val_vv(:,i);
    
    % First fundamental form
    E = u'*u;
    F = u'*v;
    G = v'*v;
    
    % Second funcamental form
    n = cross(u,v);
    n = n / norm(n);
    L = uu'*n;
    M = uv'*n;
    N = vv'*n;
    
    all_kappa(i) = (L*N-M^2) / (E*G - F^2);
end
