function test_salzmann
%TEST_SALZMANN Test of the Salzmann-related code of RIS.
%
% (c)2010, Florent Brunet

% RIS is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% RIS is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
% or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
% for more details.
% 
% You should have received a copy of the GNU General Public License along
% with this program; if not, write to the Free Software Foundation, Inc.,
% 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
tic
close all;

%% Load the data

%tmp = load('paper_creased.mat');
tmp = load('paper_sc.mat');
% corresp_2d_tpl = tmp.corresp_2d_tpl;
% corresp_3d = tmp.corresp_3d;
% corresp_2d_img = tmp.corresp_2d_img;
cam_mat_img = tmp.cam_mat_img;
mat_intr = cam_mat_img(:,1:3);
mesh_3d = tmp.mesh_3d;
mesh_2d_tpl = tmp.mesh_2d_tpl;
mesh_2d_img = tmp.mesh_2d_img;

rand('twister', 2); %#ok<RAND>
ampl_noise = 0.1;
npts = 144;
ind_corresp = randperm(size(mesh_2d_tpl,2));
ind_corresp = ind_corresp(1:npts);
corresp_2d_tpl = mesh_2d_tpl(:,ind_corresp) + ampl_noise*rand(2,npts)-ampl_noise/2;
corresp_3d = mesh_3d(:,ind_corresp);
corresp_2d_img = mesh_2d_img(:,ind_corresp) + ampl_noise*rand(2,npts)-ampl_noise/2;


umin = min(min(mesh_2d_tpl(1,:)), min(corresp_2d_tpl(1,:))) - 1;
umax = max(max(mesh_2d_tpl(1,:)), max(corresp_2d_tpl(1,:))) + 1;
vmin = min(min(mesh_2d_tpl(2,:)), min(corresp_2d_tpl(2,:))) - 1;
vmax = max(max(mesh_2d_tpl(2,:)), max(corresp_2d_tpl(2,:))) + 1;
npts_mesh = 10;
[tri tri_x tri_y tri_z] = ris_create_tri_mesh(umin, umax, 10, vmin, vmax, npts_mesh);

% figure
% trimesh(tri, tri_x, tri_y, tri_z, 'facecolor', 'none');
% view(0, 90);
% hold on;
% plot(corresp_2d_tpl(1,:), corresp_2d_tpl(2,:), '.');
% grid off;



[b1 b2 b3 ind_tri] = ris_cart_to_bary(tri, tri_x, tri_y, tri_z, ... 
    corresp_2d_tpl(1,:), corresp_2d_tpl(2,:), ones(size(corresp_2d_tpl(2,:))));

[inext_pairs dist_pairs] = ris_make_inext_pairs(tri_x, tri_y, 32);

[depth, x, y, z] = ris_salzmann_err( ...
                        tri, npts_mesh*npts_mesh, ...
                        ind_tri, b1, b2, b3, ...
                        corresp_2d_img, ...
                        cam_mat_img, ...
                        inext_pairs, dist_pairs+0.01);

%[depths, bounds3D] = ris_socp2(corresp_2d_tpl, corresp_2d_img, cam_mat_img, 0.01);

%% Display
figure(1);

% Ground truth 3D points
plot3(corresp_3d(1,:), corresp_3d(2,:), corresp_3d(3,:), 'k.');
daspect([1 1 1]);
%set(gca, 'xlim', [-70 105], 'ylim', [-110 110], 'zlim', [860 1030]);
hold on;

% Ground truth surface
gt_x = reshape(mesh_3d(1,:), 100, 100);
gt_y = reshape(mesh_3d(2,:), 100, 100);
gt_z = reshape(mesh_3d(3,:), 100, 100);
idx = 1:3:100;
mesh(gt_x(idx,idx), gt_y(idx,idx), gt_z(idx,idx), 'facecolor', 'none', 'edgecolor', 0.5*ones(1,3));

% Salzmann points
plot3(x, y, z, 'bo');

% Salzmann mesh
trimesh(tri, x, y, z, 'facecolor', 'none');

%plot3(bounds3D(1,:), bounds3D(2,:), bounds3D(3,:), 'r*');
